import unittest
from dataclasses import dataclass
from typing import Literal

from autofaker import Autodata

string_literal = Literal["pending", "approved", "rejected"]
number_literal = Literal[1, 2, 3]


class AnonymousStringLiteralTypeTestCase(unittest.TestCase):
    def test_create_literal_returns_not_none(self):
        data = Autodata.create(string_literal)
        self.assertIsNotNone(data)

    def test_create_literal_returns_valid_value(self):
        data = Autodata.create(string_literal)
        self.assertIn(data, ["pending", "approved", "rejected"])


class AnonymousNumberLiteralTypeTestCase(unittest.TestCase):
    def test_create_literal_returns_not_none(self):
        data = Autodata.create(number_literal)
        self.assertIsNotNone(data)

    def test_create_literal_returns_valid_value(self):
        data = Autodata.create(number_literal)
        self.assertIn(data, [1, 2, 3])


@dataclass
class DataClassWithLiteral:
    id: int
    status: string_literal
    number: number_literal


class AnonymousDataClassWithLiteralTypeTestCase(unittest.TestCase):
    def test_create_literal_returns_not_none(self):
        data = Autodata.create(DataClassWithLiteral)
        self.assertIsNotNone(data)

    def test_create_literal_returns_valid_value(self):
        data = Autodata.create(DataClassWithLiteral)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])

    def test_create_literal_returns_fake_value(self):
        data = Autodata.create(DataClassWithLiteral, use_fake_data=True)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])


class SimpleClassWithLiteral:
    id = -1
    text = "test"
    status: string_literal
    number: number_literal


class NestedClassWithLiteral:
    id = -1
    text = "test"
    status: string_literal
    number: number_literal
    inner = SimpleClassWithLiteral()


class DoubleNestedClassWithLiteral:
    id = -1
    text = "test"
    status: string_literal
    number: number_literal
    inner = NestedClassWithLiteral()


class AnonymousSimpleClassWithLiteralTestCase(unittest.TestCase):
    def test_create_simple_class_with_literal_returns_not_none(self):
        data = Autodata.create(SimpleClassWithLiteral)
        self.assertIsNotNone(data)

    def test_create_simple_class_with_literal_returns_instance(self):
        data = Autodata.create(SimpleClassWithLiteral)
        self.assertIsInstance(data, SimpleClassWithLiteral)

    def test_create_simple_class_with_literal_returns_valid_values(self):
        data = Autodata.create(SimpleClassWithLiteral)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])

    def test_create_simple_class_with_literal_returns_non_default(self):
        data = Autodata.create(SimpleClassWithLiteral)
        default = SimpleClassWithLiteral()
        self.assertNotEqual(data.id, default.id)
        self.assertNotEqual(data.text, default.text)


class AnonymousNestedClassWithLiteralTestCase(unittest.TestCase):
    def test_create_nested_class_with_literal_returns_not_none(self):
        data = Autodata.create(NestedClassWithLiteral)
        self.assertIsNotNone(data)

    def test_create_nested_class_with_literal_returns_instance(self):
        data = Autodata.create(NestedClassWithLiteral)
        self.assertIsInstance(data, NestedClassWithLiteral)

    def test_create_nested_class_with_literal_returns_valid_values(self):
        data = Autodata.create(NestedClassWithLiteral)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])
        self.assertIn(data.inner.status, ["pending", "approved", "rejected"])
        self.assertIn(data.inner.number, [1, 2, 3])

    def test_create_nested_class_with_literal_returns_non_default(self):
        data = Autodata.create(NestedClassWithLiteral)
        default = NestedClassWithLiteral()
        self.assertNotEqual(data.id, default.id)
        self.assertNotEqual(data.text, default.text)
        self.assertNotEqual(data.inner.id, default.inner.id)
        self.assertNotEqual(data.inner.text, default.inner.text)


class AnonymousDoubleNestedClassWithLiteralTestCase(unittest.TestCase):
    def test_create_double_nested_class_with_literal_returns_not_none(self):
        data = Autodata.create(DoubleNestedClassWithLiteral)
        self.assertIsNotNone(data)

    def test_create_double_nested_class_with_literal_returns_instance(self):
        data = Autodata.create(DoubleNestedClassWithLiteral)
        self.assertIsInstance(data, DoubleNestedClassWithLiteral)

    def test_create_double_nested_class_with_literal_returns_valid_values(self):
        data = Autodata.create(DoubleNestedClassWithLiteral)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])
        self.assertIn(data.inner.status, ["pending", "approved", "rejected"])
        self.assertIn(data.inner.number, [1, 2, 3])
        self.assertIn(data.inner.inner.status, ["pending", "approved", "rejected"])
        self.assertIn(data.inner.inner.number, [1, 2, 3])

    def test_create_double_nested_class_with_literal_returns_non_default(self):
        data = Autodata.create(DoubleNestedClassWithLiteral)
        default = DoubleNestedClassWithLiteral()
        self.assertNotEqual(data.id, default.id)
        self.assertNotEqual(data.text, default.text)
        self.assertNotEqual(data.inner.id, default.inner.id)
        self.assertNotEqual(data.inner.text, default.inner.text)
        self.assertNotEqual(data.inner.inner.id, default.inner.inner.id)
        self.assertNotEqual(data.inner.inner.text, default.inner.inner.text)


class CreateManySimpleClassWithLiteralTestCase(unittest.TestCase):
    def test_create_many_simple_class_with_literal_returns_not_none(self):
        data = Autodata.create_many(SimpleClassWithLiteral)
        self.assertIsNotNone(data)

    def test_create_many_simple_class_with_literal_returns_list(self):
        data = Autodata.create_many(SimpleClassWithLiteral)
        self.assertIsInstance(data, list)

    def test_create_many_simple_class_with_literal_returns_valid_values(self):
        data = Autodata.create_many(SimpleClassWithLiteral)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])

    def test_create_many_simple_class_with_literal_returns_non_default(self):
        data = Autodata.create_many(SimpleClassWithLiteral)
        default = SimpleClassWithLiteral()
        for item in data:
            self.assertNotEqual(item.id, default.id)
            self.assertNotEqual(item.text, default.text)


class CreateManyNestedClassWithLiteralTestCase(unittest.TestCase):
    def test_create_many_nested_class_with_literal_returns_not_none(self):
        data = Autodata.create_many(NestedClassWithLiteral)
        self.assertIsNotNone(data)

    def test_create_many_nested_class_with_literal_returns_list(self):
        data = Autodata.create_many(NestedClassWithLiteral)
        self.assertIsInstance(data, list)

    def test_create_many_nested_class_with_literal_returns_valid_values(self):
        data = Autodata.create_many(NestedClassWithLiteral)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])
            self.assertIn(item.inner.status, ["pending", "approved", "rejected"])
            self.assertIn(item.inner.number, [1, 2, 3])

    def test_create_many_nested_class_with_literal_returns_non_default(self):
        data = Autodata.create_many(NestedClassWithLiteral)
        default = NestedClassWithLiteral()
        for item in data:
            self.assertNotEqual(item.id, default.id)
            self.assertNotEqual(item.text, default.text)
            self.assertNotEqual(item.inner.id, default.inner.id)
            self.assertNotEqual(item.inner.text, default.inner.text)


class CreateManyDoubleNestedClassWithLiteralTestCase(unittest.TestCase):
    def test_create_many_double_nested_class_with_literal_returns_not_none(self):
        data = Autodata.create_many(DoubleNestedClassWithLiteral)
        self.assertIsNotNone(data)

    def test_create_many_double_nested_class_with_literal_returns_list(self):
        data = Autodata.create_many(DoubleNestedClassWithLiteral)
        self.assertIsInstance(data, list)

    def test_create_many_double_nested_class_with_literal_returns_valid_values(self):
        data = Autodata.create_many(DoubleNestedClassWithLiteral)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])
            self.assertIn(item.inner.status, ["pending", "approved", "rejected"])
            self.assertIn(item.inner.number, [1, 2, 3])
            self.assertIn(item.inner.inner.status, ["pending", "approved", "rejected"])
            self.assertIn(item.inner.inner.number, [1, 2, 3])

    def test_create_many_double_nested_class_with_literal_returns_non_default(self):
        data = Autodata.create_many(DoubleNestedClassWithLiteral)
        default = DoubleNestedClassWithLiteral()
        for item in data:
            self.assertNotEqual(item.id, default.id)
            self.assertNotEqual(item.text, default.text)
            self.assertNotEqual(item.inner.id, default.inner.id)
            self.assertNotEqual(item.inner.text, default.inner.text)
            self.assertNotEqual(item.inner.inner.id, default.inner.inner.id)
            self.assertNotEqual(item.inner.inner.text, default.inner.inner.text)


class CreateManyWithSizeTestCase(unittest.TestCase):
    def test_create_many_simple_class_with_literal_with_size(self):
        size = 5
        data = Autodata.create_many(SimpleClassWithLiteral, size)
        self.assertEqual(len(data), size)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])

    def test_create_many_nested_class_with_literal_with_size(self):
        size = 3
        data = Autodata.create_many(NestedClassWithLiteral, size)
        self.assertEqual(len(data), size)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])
            self.assertIn(item.inner.status, ["pending", "approved", "rejected"])
            self.assertIn(item.inner.number, [1, 2, 3])

    def test_create_many_double_nested_class_with_literal_with_size(self):
        size = 2
        data = Autodata.create_many(DoubleNestedClassWithLiteral, size)
        self.assertEqual(len(data), size)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])
            self.assertIn(item.inner.status, ["pending", "approved", "rejected"])
            self.assertIn(item.inner.number, [1, 2, 3])
            self.assertIn(item.inner.inner.status, ["pending", "approved", "rejected"])
            self.assertIn(item.inner.inner.number, [1, 2, 3])


class FakeDataWithLiteralTestCase(unittest.TestCase):
    def test_create_simple_class_with_literal_with_fake_data(self):
        data = Autodata.create(SimpleClassWithLiteral, use_fake_data=True)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])

    def test_create_nested_class_with_literal_with_fake_data(self):
        data = Autodata.create(NestedClassWithLiteral, use_fake_data=True)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])
        self.assertIn(data.inner.status, ["pending", "approved", "rejected"])
        self.assertIn(data.inner.number, [1, 2, 3])

    def test_create_double_nested_class_with_literal_with_fake_data(self):
        data = Autodata.create(DoubleNestedClassWithLiteral, use_fake_data=True)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])
        self.assertIn(data.inner.status, ["pending", "approved", "rejected"])
        self.assertIn(data.inner.number, [1, 2, 3])
        self.assertIn(data.inner.inner.status, ["pending", "approved", "rejected"])
        self.assertIn(data.inner.inner.number, [1, 2, 3])

    def test_create_many_with_literal_with_fake_data(self):
        size = 3
        data = Autodata.create_many(SimpleClassWithLiteral, size, use_fake_data=True)
        self.assertEqual(len(data), size)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])

    def test_create_local_class_with_literal(self):
        class LocalWithLiteral:
            id = -1
            status: string_literal
            number: number_literal

        data = Autodata.create(LocalWithLiteral)
        self.assertIsNotNone(data)
        self.assertIsInstance(data, LocalWithLiteral)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])

    def test_create_local_nested_class_with_literal(self):
        class LocalInner:
            id = -1
            status: string_literal
            number: number_literal

        class LocalOuter:
            id = -1
            status: string_literal
            number: number_literal
            inner = LocalInner()

        data = Autodata.create(LocalOuter)
        self.assertIsNotNone(data)
        self.assertIsInstance(data, LocalOuter)
        self.assertIn(data.status, ["pending", "approved", "rejected"])
        self.assertIn(data.number, [1, 2, 3])
        self.assertIn(data.inner.status, ["pending", "approved", "rejected"])
        self.assertIn(data.inner.number, [1, 2, 3])

    def test_create_many_local_class_with_literal(self):
        class LocalWithLiteral:
            id = -1
            status: string_literal
            number: number_literal

        size = 3
        data = Autodata.create_many(LocalWithLiteral, size)
        self.assertIsNotNone(data)
        self.assertIsInstance(data, list)
        self.assertEqual(len(data), size)
        for item in data:
            self.assertIn(item.status, ["pending", "approved", "rejected"])
            self.assertIn(item.number, [1, 2, 3])



import random
from typing import get_args, get_origin, Literal

from autofaker.base import TypeDataGeneratorBase


def is_literal_type(t):
    origin = get_origin(t)
    if origin is None:
        return False
    return origin is Literal


class LiteralGenerator(TypeDataGeneratorBase):
    def __init__(self, t):
        if not is_literal_type(t):
            raise ValueError(f"Expected Literal type, got {t}")
        self.allowed_values = get_args(t)
        if not self.allowed_values:
            raise ValueError("Literal type must have at least one value")

    def generate(self):
        return random.choice(self.allowed_values)



import unittest
from abc import abstractmethod
from typing import Type

from autofaker import Autodata


class CreateTestCase(unittest.TestCase):
    @abstractmethod
    def getType(self) -> Type:
        pass

    def test_create_returns_not_none(self):
        if self.getType() is not None:
            self.assertIsNotNone(Autodata.create(self.getType()))

    def test_create_returns_not_type(self):
        if self.getType() is not None:
            self.assertNotIsInstance(Autodata.create(self.getType()), type)

    def test_create_returns_not_default(self):
        if (self.getType() is not None
            and not bool
            and not range):
            self.assertNotEqual(Autodata.create(self.getType()), self.getType()())


class AnonymousIntegerTestCase(CreateTestCase):
    def getType(self) -> Type[int]:
        return int


class AnonymousStringTestCase(CreateTestCase):
    def getType(self) -> Type[str]:
        return str


class AnonymousFloatTestCase(CreateTestCase):
    def getType(self) -> Type[float]:
        return float


class AnonymousBooleanTestCase(CreateTestCase):
    def getType(self) -> Type[bool]:
        return bool


class AnonymousComplexTestCase(CreateTestCase):
    def getType(self) -> Type[complex]:
        return complex


class AnonymousRangeTestCase(CreateTestCase):
    def getType(self) -> Type[range]:
        return range


class AnonymousBytesTestCase(CreateTestCase):
    def getType(self) -> Type[bytes]:
        return bytes


class AnonymousByteArrayTestCase(CreateTestCase):
    def getType(self) -> Type[bytearray]:
        return bytearray


class AnonymousMemoryViewTestCase(CreateTestCase):
    def getType(self) -> Type[memoryview]:
        return memoryview


class AnonymousSetTestCase(CreateTestCase):
    def getType(self) -> Type[set]:
        return set


class AnonymousFrozensetTestCase(CreateTestCase):
    def getType(self) -> Type[frozenset]:
        return frozenset



"""
Provides classes for anonymous object creation to help minimize
the setup/arrange phase when writing unit tests
"""

import typing
from typing import List

from autofaker.dataframe import PandasDataFrameGenerator
from autofaker.generator import TypeDataGenerator


class Autodata:
    """
    Provides anonymous object creation functions to help minimize
    the setup/arrange phase when writing unit tests
    """

    @staticmethod
    def create(t, use_fake_data: bool = False):
        """
        Creates an anonymous variable of the requested type

        :param type t: type
            The type to generate data for

        :param use_fake_data: bool
            Set this to True to use Faker to generate data
            otherwise False to generate anonymous data
        """
        return TypeDataGenerator.create(t, use_fake_data=use_fake_data).generate()

    @staticmethod
    def create_many(t, size: int = 3, use_fake_data: bool = False) -> List[typing.Any]:
        """
        Creates a list of anonymous variables of the requested
        type using the specified length (default 3)

        :param type t: (type)
            The type to generate data for

        :param size: (int)
            The number of items to generate (default 3)

        :param use_fake_data: (bool)
            Set this to True to use Faker to generate data,
            otherwise False to generate anonymous data
        """
        items = []
        for _ in range(size):
            items.append(Autodata.create(t, use_fake_data=use_fake_data))
        return items

    @staticmethod
    def create_pandas_dataframe(t, rows: int = 3, use_fake_data: bool = False):
        """
        Create a Pandas DataFrame containing anonymous data
        with the specified number of rows (default 3)

        :param type t: object
            The class that represents the DataFrame.
            This can be a plain old class or a @dataclass

        :param type rows: int
            The number of rows to generate for the DataFrame (default 3)

        :param use_fake_data: bool
            Set this to True to use Faker to generate data,
            otherwise False to generate anonymous data
        """
        return PandasDataFrameGenerator(t, rows, use_fake_data=use_fake_data).generate()
